@@ -1,19 +1,15 @@
 package XML::SemanticDiff;
+our $AUTHORITY = 'cpan:SHLOMIF';
 
 use strict;
 use warnings;
 
-use vars qw/$VERSION/;
+use 5.012;
 
-$VERSION = '1.0004';
+our $VERSION = '1.0006';
 
 use XML::Parser;
 
-=head1 NAME
-
-XML::SemanticDiff - Perl extension for comparing XML documents.
-
-=cut
 
 sub new {
     my ($proto, %args) = @_;
@@ -63,7 +59,7 @@ sub read_xml {
                 ? $p->parsefile($xml_specifier)
                 : $p->parse($xml_specifier)
                 ;
-        
+
         $self->{path_finder_obj} = undef;
 
         return $ret;
@@ -73,7 +69,7 @@ sub read_xml {
 sub _same_namespace
 {
     my ($self, $to, $from) = @_;
-    
+
     my $t_e = exists($to->{NamespaceURI});
     my $f_e = exists($from->{NamespaceURI});
     if (!$t_e && !$f_e)
@@ -97,10 +93,10 @@ sub _match_xpath {
     my @f_way = split /\//, $flat_name;
     for my $i (0..$#x_way) {
         $x_way[$i]=~s/.*?://g;
-    }           
+    }
     for my $i (0..$#f_way) {
         $f_way[$i]=~s/\[.*?\]$//g;
-    }   
+    }
     return 0 if $#x_way > $#f_way;
     for my $i (0..$#x_way) {
         if ($x_way[$i] ne $f_way[$i]) {
@@ -112,14 +108,14 @@ sub _match_xpath {
 
 # Okay, it's pretty basic...
 #
-# We flatten each doc tree to a Perl hash where the keys are "fully qualified" 
+# We flatten each doc tree to a Perl hash where the keys are "fully qualified"
 # XPath expressions (/root[1]/element[3]) that represent the unique location
-# of each XML element, then compare the two hashes. 
+# of each XML element, then compare the two hashes.
 #
 # Just loop over all the elements of the first hash- if the same key exists
 # in the second, you compare the text and attributes and delete it. Any
 # keys not found in the second hash are declared 'missing', and any keys leftover
-# in the second hash after looping through the elements in the first are 'rogues'. 
+# in the second hash after looping through the elements in the first are 'rogues'.
 
 sub compare {
     my $self = shift;
@@ -157,30 +153,30 @@ sub compare {
 
             # element value test
             unless ($from_doc->{$element}->{TextChecksum} eq $to_doc->{$element}->{TextChecksum}) {
-                push (@warnings, $handler->element_value($element, 
-                                                         $to_doc->{$element}, 
+                push (@warnings, $handler->element_value($element,
+                                                         $to_doc->{$element},
                                                          $from_doc->{$element}))
                           if $handler->can('element_value');
             }
-        
+
             # namespace test
             unless ($self->_same_namespace($from_doc->{$element},$to_doc->{$element})) {
-                push (@warnings, $handler->namespace_uri($element, 
-                                                         $to_doc->{$element}, 
+                push (@warnings, $handler->namespace_uri($element,
+                                                         $to_doc->{$element},
                                                          $from_doc->{$element}))
                            if $handler->can('namespace_uri');
             }
-    
+
             # attribute tests
             foreach my $attr (keys(%{$from_doc->{$element}->{Attributes}})) {
- 
+
                  # attr existence check
                  if (defined ($to_doc->{$element}->{Attributes}->{$attr})) {
 
                      # attr value test
                      if ($to_doc->{$element}->{Attributes}->{$attr} ne $from_doc->{$element}->{Attributes}->{$attr}){
-                        push (@warnings, $handler->attribute_value($attr, 
-                                                                   $element, 
+                        push (@warnings, $handler->attribute_value($attr,
+                                                                   $element,
                                                                    $to_doc->{$element},
                                                                    $from_doc->{$element}))
                               if $handler->can('attribute_value');
@@ -188,45 +184,46 @@ sub compare {
                      delete $to_doc->{$element}->{Attributes}->{$attr};
                  }
                  else {
-                     push (@warnings, $handler->missing_attribute($attr, 
-                                                                  $element, 
-                                                                  $to_doc->{$element}, 
+                     push (@warnings, $handler->missing_attribute($attr,
+                                                                  $element,
+                                                                  $to_doc->{$element},
                                                                   $from_doc->{$element}))
-                           if $handler->can('missing_attribute'); 
-                 }                
+                           if $handler->can('missing_attribute');
+                 }
             }
 
             # rogue attrs
             foreach my $leftover (keys(%{$to_doc->{$element}->{Attributes}})) {
-                push (@warnings, $handler->rogue_attribute($leftover, 
-                                                           $element, 
-                                                           $to_doc->{$element}, 
+                push (@warnings, $handler->rogue_attribute($leftover,
+                                                           $element,
+                                                           $to_doc->{$element},
                                                            $from_doc->{$element}))
                      if $handler->can('rogue_attribute');
             }
-            
+
             delete $to_doc->{$element};
         }
-        else {  
+        else {
             push (@warnings, $handler->missing_element($element, $from_doc->{$element}))
-                      if $handler->can('missing_element');          
+                      if $handler->can('missing_element');
         }
     }
 
     # rogue elements
     foreach my $leftover ( keys (%$to_doc) ) {
-        push (@warnings, $handler->rogue_element($leftover, $to_doc->{$leftover})) 
+        push (@warnings, $handler->rogue_element($leftover, $to_doc->{$leftover}))
              if $handler->can('rogue_element');
-    }                 
+    }
 
     push (@warnings, $handler->final($self)) if $handler->can('final');
-            
+
     return @warnings;
 }
 
 1;
 
 package XML::SemanticDiff::PathFinder;
+our $AUTHORITY = 'cpan:SHLOMIF';
 
 foreach my $func (qw(StartTag EndTag Text StartDocument EndDocument PI))
 {
@@ -240,6 +237,7 @@ foreach my $func (qw(StartTag EndTag Text StartDocument EndDocument PI))
 }
 
 package XML::SemanticDiff::PathFinder::Obj;
+our $AUTHORITY = 'cpan:SHLOMIF';
 
 use strict;
 
@@ -262,7 +260,7 @@ foreach my $accessor (qw(descendents char_accumulator doc
     };
 }
 
-# PI_position_index is the position index for the PI's below - the processing 
+# PI_position_index is the position index for the PI's below - the processing
 # instructions.
 
 sub new {
@@ -285,7 +283,7 @@ sub StartTag {
 
 
     my %attrs = %_;
-            
+
     my @context = $expat->context;
     my $context_length = scalar (@context);
     my $parent = $context[$context_length -1];
@@ -293,29 +291,29 @@ sub StartTag {
 
     my $last_ctx_elem = $self->xml_context()->[-1] || { position_index => {}};
 
-    push @{$self->xml_context()}, 
+    push @{$self->xml_context()},
         {
-            element => "$element", 
+            element => "$element",
             'index' => ++$last_ctx_elem->{position_index}->{"$element"},
             position_index => {},
         };
 
     my $test_context;
- 
+
     # if (@context){
     #     $test_context = '/' . join ('/', map { $_ . '[' . $position_index->{$_} . ']' } @context);
-    # }   
-        
+    # }
+
     # $test_context .= '/' . $element . '[' . $position_index->{$element} . ']';
 
     $test_context = $self->_calc_test_context();
 
-    $self->doc()->{$test_context} = 
+    $self->doc()->{$test_context} =
     {
         NamespaceURI => ($expat->namespace($element) || ""),
         Attributes   => \%attrs,
         ($self->opts()->{keeplinenums}
-            ? ( TagStart => $expat->current_line) 
+            ? ( TagStart => $expat->current_line)
             : ()
         ),
     };
@@ -325,43 +323,57 @@ sub _calc_test_context
 {
     my $self = shift;
 
-    return 
-        join("", 
-            map { "/". $_->{'element'} . "[" . $_->{'index'} . "]" } 
+    return
+        join("",
+            map { "/". $_->{'element'} . "[" . $_->{'index'} . "]" }
             @{$self->xml_context()}
         );
 }
 
 sub EndTag {
     my ($self, $expat, $element) = @_;
-    
+
     my @context = $expat->context;
 
     # if (@context){
     #    $test_context = '/' . join ('/', map { $_ . '[' . $position_index->{$_} . ']' } @context);
-    #} 
+    #}
     # $test_context .= '/' . $element . '[' . $position_index->{$element} . ']';
 
     my $test_context = $self->_calc_test_context();
 
     my $text;
-    if ( defined( $self->char_accumulator()->{$element} )) { 
+    if ( defined( $self->char_accumulator()->{$element} )) {
         $text = $self->char_accumulator()->{$element};
         delete $self->char_accumulator()->{$element};
     }
-    $text ||= 'o';
-    
+    # This isn't the correct thing to do.  If the before or after element
+    # had and 'o' and the other was undef, we would fail to find any differences
+    # Instead, when a value is undef we should be setting the the checksum
+    # to the value for an empty string since undef and empty string for a
+    # element are the same (<element /> vs <element></element>)
+    #$text ||= 'o';
+
 #    warn "text is '$text' \n";
 #    my $ctx = Digest::MD5->new;
 #    $ctx->add("$text");
 #    $self->doc()->{"$test_context"}->{TextChecksum} = $ctx->b64digest;
 
-    $self->doc()->{"$test_context"}->{TextChecksum} = md5_base64(encode_utf8("$text"));
+    # In XML, a null(undef) value and an empty string should be treaded the same.
+    # Therefore, when the element is undef, we should set the TextChecksum to the same
+    # as an empty string.
+    $self->doc()->{"$test_context"}->{TextChecksum} =
+        md5_base64(
+            encode_utf8(
+                (defined $text) ? "$text" : ""
+            )
+        );
+
     if ($self->opts()->{keepdata}) {
         $self->doc()->{"$test_context"}->{CData} = $text;
     }
-    
-    
+
+
     if (defined ( $self->descendents()->{$element})) {
         my $seen = {};
         foreach my $child (@{$self->descendents()->{$element}}) {
@@ -369,7 +381,7 @@ sub EndTag {
             $seen->{$child}++;
         }
     }
-    
+
     $self->doc()->{"$test_context"}->{TagEnd} = $expat->current_line if $self->opts()->{keeplinenums};
 
     pop(@{$self->xml_context()});
@@ -378,17 +390,19 @@ sub EndTag {
 sub Text {
     my $self = shift;
     my $expat = shift;
-    
+
     my $element = $expat->current_element;
     my $char = $_;
-    
+
     $char =~ s/^\s*//;
     $char =~ s/\s*$//;
     $char =~ s/\s+/ /g;
-    $self->char_accumulator()->{$element} .= $char if $char;
-    
+    # We should add any character that isn't undef, so check
+    # for defined here instead of checking if the value is true
+    $self->char_accumulator()->{$element} .= $char if defined($char);
+
 }
-        
+
 sub StartDocument {
     my $self = shift;
     my $expat = shift;
@@ -399,7 +413,7 @@ sub StartDocument {
     $self->xml_context([]);
     $self->PI_position_index({});
 }
-        
+
 sub EndDocument {
     my $self = shift;
 
@@ -422,8 +436,8 @@ sub PI {
         {
             Attributes => ($attrs || {}),
             TextChecksum => "1",
-            NamespaceURI => "", 
-            ( $self->opts()->{keeplinenums} 
+            NamespaceURI => "",
+            ( $self->opts()->{keeplinenums}
             ? (
                 TagStart => $expat->current_line(),
                 TagEnd => $expat->current_line(),
@@ -431,10 +445,21 @@ sub PI {
             : ()
             ),
         };
-}   
+}
 
 1;
 
+__END__
+
+=pod
+
+=head1 NAME
+
+XML::SemanticDiff - Perl extension for comparing XML documents.
+
+=head1 VERSION
+
+version 1.0006
 
 =head1 SYNOPSIS
 
@@ -458,6 +483,10 @@ sub PI {
 XML::SematicDiff provides a way to compare the contents and structure of two XML documents. By default, it returns a list of
 hashrefs where each hashref describes a single difference between the two docs.
 
+=head1 VERSION
+
+version 1.0006
+
 =head1 METHODS
 
 =head2 $obj->new([%options])
@@ -472,29 +501,29 @@ The new() method recognizes the following options:
 
 When this option is enabled XML::SemanticDiff will add the 'startline' and 'endline' properties (containing the line numbers
 for the reported element's start tag and end tag) to each warning. For attribute events these numbers reflect the start and
-end tags of the element which contains that attribute. 
+end tags of the element which contains that attribute.
 
 =item * keepdata
 
 When this option is enabled XML::SemanticDiff will add the 'old_value' and 'new_value' properties to each warning. These
-properties contain, surprisingly, the old and new values for the element or attribute being reported. 
+properties contain, surprisingly, the old and new values for the element or attribute being reported.
 
 In the case of missing elements or attributes (those in the first document, not in the second) only the 'old_value' property
 will be defined. Similarly, in the case of rogue elements or attributes (those in the second document but not in the
-first) only the 'new_value' property will be defined. 
+first) only the 'new_value' property will be defined.
 
 Note that using this option will greatly increase the amount of memory used by your application.
 
 =item * diffhandler
 
 Taking a blessed object as it's sole argument, this option provides a way to hook the basic semantic diff engine into your own
-custom handler class. 
+custom handler class.
 
 Please see the section on 'CUSTOM HANDLERS' below.
 
-=item  * ignorexpath
+=item * ignorexpath
 
-This option takes array of strings as argument. Strings are interpreted as simple xpath expressions. Nodes matching these expressions are ignored during comparison. All xpath expressions should be absolute (start with '/'). 
+This option takes array of strings as argument. Strings are interpreted as simple xpath expressions. Nodes matching these expressions are ignored during comparison. All xpath expressions should be absolute (start with '/').
 
 Current implementation ignores namespaces during comparison.
 
@@ -508,7 +537,7 @@ Compares the XMLs $xml1 and $xml2 . $xml1 and $xml2 can be:
 
 =item * filenames
 
-This will be considered if it is a string that does not contain newlines and 
+This will be considered if it is a string that does not contain newlines and
 exists in the filesystem.
 
 =item * the XML text itself.
@@ -529,9 +558,9 @@ for how it is determined.
 =head1 CUSTOM HANDLERS
 
 Internally, XML::SemanticDiff uses an event-based model somewhat reminiscent of SAX where the various 'semantic diff events'
-are handed off to a separate handler class to cope with the details. For most general cases where the user only cares about 
-reporting the differences between two docs, the default handler, XML::SemanticDiff::BasicHandler, will probably  
-suffice. However, it is often desirable to add side-effects to the diff process (updating datastores, widget callbacks,  
+are handed off to a separate handler class to cope with the details. For most general cases where the user only cares about
+reporting the differences between two docs, the default handler, XML::SemanticDiff::BasicHandler, will probably
+suffice. However, it is often desirable to add side-effects to the diff process (updating datastores, widget callbacks,
 etc.) and a custom handler allows you to be creative with what to do about differences between two XML documents and how
 those differences are reported back to the application through the compare() method.
 
@@ -562,9 +591,8 @@ mixed-content elements (those containing both text and child elements mixed toge
 =head2 namespace_uri($self, $element, $todoc_element_properties, $fromdoc_element_properties)
 
 The C<namespace_uri> method handles case where the XML namespace URI differs between a given element in the two
-documents. Note that the namespace URI is checked, not the element prefixes since <foo:element/> <bar:element/> and <element/> 
+documents. Note that the namespace URI is checked, not the element prefixes since <foo:element/> <bar:element/> and <element/>
 are all considered equivalent as long as they are bound to the same namespace URI.
- 
 
 =head2 rogue_attribute($self, $attr_name, $element, $todoc_element_properties)
 
@@ -587,7 +615,7 @@ The C<final> method is called immediately after the two document HASHes are comp
 copy of the XML::SemanticDiff object as it's sole argument.
 
 Note that if a given method is not implemented in your custom handler class, XML::SemanticDiff will not complain; but it means
-that all of those events will be silently ignored. Consider yourself warned. 
+that all of those events will be silently ignored. Consider yourself warned.
 
 =head1 AUTHOR
 
@@ -597,8 +625,8 @@ Further Maintained by Shlomi Fish, L<http://www.shlomifish.org/> .
 
 =head1 COPYRIGHT
 
-Copyright (c) 2000 Kip Hampton. All rights reserved. This program is 
-free software; you can redistribute it and/or modify it under the same terms 
+Copyright (c) 2000 Kip Hampton. All rights reserved. This program is
+free software; you can redistribute it and/or modify it under the same terms
 as Perl itself.
 
 Shlomi Fish hereby disclaims any implicit or explicit copyrights on this
@@ -606,11 +634,148 @@ software.
 
 =head1 LICENSE
 
-This program is free software; you can redistribute it and/or modify it under 
+This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
 =head1 SEE ALSO
 
 perl(1).
 
+=head1 AUTHOR
+
+Shlomi Fish <shlomif@cpan.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2001 by Kim Hampton.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=head1 BUGS
+
+Please report any bugs or feature requests on the bugtracker website
+L<http://rt.cpan.org/Public/Dist/Display.html?Name=XML-SemanticDiff> or by
+email to
+L<bug-xml-semanticdiff@rt.cpan.org|mailto:bug-xml-semanticdiff@rt.cpan.org>.
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
+=for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
+
+=head1 SUPPORT
+
+=head2 Perldoc
+
+You can find documentation for this module with the perldoc command.
+
+  perldoc XML::SemanticDiff
+
+=head2 Websites
+
+The following websites have more information about this module, and may be of help to you. As always,
+in addition to those websites please use your favorite search engine to discover more resources.
+
+=over 4
+
+=item *
+
+MetaCPAN
+
+A modern, open-source CPAN search engine, useful to view POD in HTML format.
+
+L<http://metacpan.org/release/XML-SemanticDiff>
+
+=item *
+
+Search CPAN
+
+The default CPAN search engine, useful to view POD in HTML format.
+
+L<http://search.cpan.org/dist/XML-SemanticDiff>
+
+=item *
+
+RT: CPAN's Bug Tracker
+
+The RT ( Request Tracker ) website is the default bug/issue tracking system for CPAN.
+
+L<https://rt.cpan.org/Public/Dist/Display.html?Name=XML-SemanticDiff>
+
+=item *
+
+AnnoCPAN
+
+The AnnoCPAN is a website that allows community annotations of Perl module documentation.
+
+L<http://annocpan.org/dist/XML-SemanticDiff>
+
+=item *
+
+CPAN Ratings
+
+The CPAN Ratings is a website that allows community ratings and reviews of Perl modules.
+
+L<http://cpanratings.perl.org/d/XML-SemanticDiff>
+
+=item *
+
+CPAN Forum
+
+The CPAN Forum is a web forum for discussing Perl modules.
+
+L<http://cpanforum.com/dist/XML-SemanticDiff>
+
+=item *
+
+CPANTS
+
+The CPANTS is a website that analyzes the Kwalitee ( code metrics ) of a distribution.
+
+L<http://cpants.cpanauthors.org/dist/XML-SemanticDiff>
+
+=item *
+
+CPAN Testers
+
+The CPAN Testers is a network of smokers who run automated tests on uploaded CPAN distributions.
+
+L<http://www.cpantesters.org/distro/X/XML-SemanticDiff>
+
+=item *
+
+CPAN Testers Matrix
+
+The CPAN Testers Matrix is a website that provides a visual overview of the test results for a distribution on various Perls/platforms.
+
+L<http://matrix.cpantesters.org/?dist=XML-SemanticDiff>
+
+=item *
+
+CPAN Testers Dependencies
+
+The CPAN Testers Dependencies is a website that shows a chart of the test results of all dependencies for a distribution.
+
+L<http://deps.cpantesters.org/?module=XML::SemanticDiff>
+
+=back
+
+=head2 Bugs / Feature Requests
+
+Please report any bugs or feature requests by email to C<bug-xml-semanticdiff at rt.cpan.org>, or through
+the web interface at L<https://rt.cpan.org/Public/Bug/Report.html?Queue=XML-SemanticDiff>. You will be automatically notified of any
+progress on the request by the system.
+
+=head2 Source Code
+
+The code is open to the world, and available for you to hack on. Please feel free to browse it and play
+with it, or whatever. If you want to contribute patches, please send me a diff or prod me to pull
+from your repository :)
+
+L<https://github.com/shlomif/perl-XML-SemanticDiff>
+
+  git clone https://github.com/shlomif/perl-XML-SemanticDiff.git
+
 =cut
